@@ -113,7 +113,7 @@ NaN = decimal.Decimal("nan")
 
 # exchanges where test_get_historical_symbol_prices is successful can be listed here
 FULL_CANDLE_HISTORY_EXCHANGES = [
-    # "ascendex",   broken since ccxt==4.4.85
+    "ascendex",
     "binance",
     "bitfinex",
     "bitstamp",
@@ -124,6 +124,8 @@ FULL_CANDLE_HISTORY_EXCHANGES = [
     "kucoin",
     "okcoin",
     "okx",
+    "myokx",
+    "okxus",
     "mexc",
     "coinbase",
     "binanceus",
@@ -156,6 +158,7 @@ TESTED_EXCHANGES = [
 DEFAULT_FUTURE_EXCHANGES = sorted(["bybit"])
 SIMULATOR_TESTED_EXCHANGES = sorted(["bitfinex", "bithumb", "bitstamp", "bitmex",
                               "hitbtc", "kraken", "poloniex", "bitso", "ndax", "upbit",
+                              "myokx", "okxus",
                               "wavesexchange",])
 
 
@@ -172,10 +175,13 @@ DEFAULT_SYMBOL_CONTRACT_SIZE = ONE
 DEFAULT_SYMBOL_POSITION_MODE = enums.PositionMode.ONE_WAY
 DEFAULT_SYMBOL_FUNDING_RATE = decimal.Decimal("0.00005")
 DEFAULT_SYMBOL_MAINTENANCE_MARGIN_RATE = decimal.Decimal("0.01")
+RETRIABLE_EXCHANGE_ERRORS_DESC: set[str] = set(os.getenv(
+    "RETRIABLE_EXCHANGE_ERRORS_DESC", 'Internal Server Error:Bad gateway:read ECONNRESET:read ETIMEDOUT'
+).split(":"))
 
 # exchange proxy
 RETRIABLE_EXCHANGE_PROXY_ERRORS_DESC: set[str] = set(os.getenv(
-    "RETRIABLE_EXCHANGE_PROXY_ERROR_DESC", "message='Service Unavailable'"
+    "RETRIABLE_EXCHANGE_PROXY_ERRORS_DESC", "message='Service Unavailable'"
 ).split(":"))
 
 # used to force margin type update before positions init (if necessary)
