@@ -119,6 +119,10 @@ class CCXTWebsocketConnector(abstract_websocket_exchange.AbstractWebsocketExchan
         self.headers = {}
         self.options = {
             "newUpdates": True,  # only get new updates from trades and ohlcv (don't return the full cached history)
+            "OHLCVLimit": trading_constants.CCXT_OHLCV_CACHE_LIMIT,
+            "tradesLimit": trading_constants.CCXT_TRADES_CACHE_LIMIT,
+            "ordersLimit": trading_constants.CCXT_ORDERS_CACHE_LIMIT,
+            "watchOrderBookLimit": trading_constants.CCXT_WATCH_ORDER_BOOK_LIMIT,
         }
         # add default options
         self.add_options(
@@ -253,7 +257,10 @@ class CCXTWebsocketConnector(abstract_websocket_exchange.AbstractWebsocketExchan
         self.client, self.is_authenticated = ccxt_client_util.create_client(
             client_class, self.exchange_manager, self.logger,
             self.options, self.headers, self.additional_config,
-            self._should_authenticate()
+            self._should_authenticate(),
+            # Always disable request counter: ws exchange will be started in a different async loop,
+            # which is not yet supported
+            allow_request_counter=False
         )
         if self.exchange_manager.exchange.is_supporting_sandbox():
             ccxt_client_util.set_sandbox_mode(self, self.exchange_manager.is_sandboxed)
@@ -620,14 +627,14 @@ class CCXTWebsocketConnector(abstract_websocket_exchange.AbstractWebsocketExchan
                     if not already_got_closed_by_user_error:
                         count_error = False
                         self.logger.debug(
-                            f"{ws_des} connection automatically closed, reconnecting in {self.LONG_RECONNECT_DELAY} "
-                            f"seconds ({err})"
+                            f"{ws_des} connection automatically closed, reconnecting in {self.LONG_RECONNECT_DELAY}"
+                            f" seconds ({err})"
                         )
                         already_got_closed_by_user_error = True
                 if count_error:
                     error_count = self._increment_error_counter(g_kwargs.get("time_frame"), err)
-                    error_message = f"Unexpected error when handling {ws_des} feed: {err} ({err.__class__.__name__}) " \
-                                    f"({error_count} times)"
+                    error_message = f"Unexpected error when handling {ws_des} feed: {err} " \
+                                    f"({err.__class__.__name__}) ({error_count} times)"
                     if error_count == 1:
                         self.logger.exception(err, True, error_message)
                     elif error_count % spamming_logs_warning_interval == 0:
@@ -642,7 +649,7 @@ class CCXTWebsocketConnector(abstract_websocket_exchange.AbstractWebsocketExchan
     def _create_task_if_necessary(self, feed, feed_callback, feed_generator, **kwargs):
         identifier = self._get_feed_identifier(feed_generator, kwargs)
         if identifier not in self.feed_tasks:
-            self.logger.debug(f"Subscribing to {feed.value} with {kwargs}")
+            self.logger.debug(f"Subscribing to {feed.value} with {kwargs} ({len(self.feed_tasks)} total feeds)")
             self.feed_tasks[identifier] = asyncio.create_task(
                 self._feed_task(feed, feed_callback, feed_generator, **kwargs)
             )
@@ -766,11 +773,22 @@ class CCXTWebsocketConnector(abstract_websocket_exchange.AbstractWebsocketExchan
         return self.EXCHANGE_FEEDS.get(Feeds.CANDLE, Feeds.UNSUPPORTED.value) != Feeds.UNSUPPORTED.value
 
     def _is_supported_pair(self, pair):
-        return pair in ccxt_client_util.get_symbols(self.client)
+        return pair in ccxt_client_util.get_symbols(self.client, True)
 
     def _is_pair_independent_feed(self, feed):
         return feed in self.PAIR_INDEPENDENT_CHANNELS
 
+    @classmethod
+    def get_feeds_count(cls, pairs, time_frames) -> int:
+        total_count = 0
+        if pairs:
+            # rounding: account for 1 feed per symbol
+            total_count = len(pairs)
+            if cls.get_exchange_feed(trading_enums.WebsocketFeeds.CANDLE):
+                # rounding: account for 1 per timeframe per symbol
+                total_count += len(pairs) * len(time_frames)
+        return total_count
+
     def _convert_book_prices_to_orders(self, book_prices_and_volumes, book_side):
         """
         Convert a book_prices format : {PRICE_1: SIZE_1, PRICE_2: SIZE_2...}
@@ -915,7 +933,8 @@ class CCXTWebsocketConnector(abstract_websocket_exchange.AbstractWebsocketExchan
                 Ectc.CLOSE.value: kline[commons_enums.PriceIndexes.IND_PRICE_CLOSE.value],
                 Ectc.LAST.value: kline[commons_enums.PriceIndexes.IND_PRICE_CLOSE.value],
                 Ectc.PREVIOUS_CLOSE.value: None,
-                Ectc.BASE_VOLUME.value: kline[commons_enums.PriceIndexes.IND_PRICE_VOL.value],
+                # do not include volume as this is not the ticker volume but the current candle volume
+                Ectc.BASE_VOLUME.value: None,
                 Ectc.TIMESTAMP.value: self.exchange.get_exchange_current_time(),
             }
             await self.push_to_channel(
