@@ -139,6 +139,8 @@ class RestExchange(abstract_exchange.AbstractExchange):
     EXCHANGE_ORDER_IMMEDIATELY_TRIGGER_ERRORS: typing.List[typing.Iterable[str]] = []
     # text content of errors due to an order that can't be cancelled on exchange (because filled or already cancelled)
     EXCHANGE_ORDER_UNCANCELLABLE_ERRORS: typing.List[typing.Iterable[str]] = []
+    # set when the exchange can allow users to pay fees in a custom currency (ex: BNB on binance)
+    LOCAL_FEES_CURRENCIES: typing.List[str] = []
 
     DEFAULT_CONNECTOR_CLASS = ccxt_connector.CCXTConnector
 
@@ -258,6 +260,8 @@ class RestExchange(abstract_exchange.AbstractExchange):
             yield
         except ccxt.InsufficientFunds as e:
             self._on_missing_funds_err(e, order_type, symbol, quantity, price, stop_price)
+        except ccxt.MarketClosed as err:
+            raise errors.MarketClosedError(html_util.get_html_summary_if_relevant(err)) from err
         except (ccxt.NotSupported, NotImplementedError) as err:
             raise errors.NotSupported(err) from err
         except (errors.AuthenticationError, ccxt.AuthenticationError) as err:
@@ -371,6 +375,25 @@ class RestExchange(abstract_exchange.AbstractExchange):
                 ) from err
             # otherwise propagate exception: this is not a situation to retry
             raise
+        except ccxt.ExchangeNotAvailable as err:
+            if not self._enable_create_order_retrier:
+                # should not retry, raise
+                raise
+            is_retriable_error = False
+            for error_message in constants.RETRIABLE_EXCHANGE_ERRORS_DESC:
+                if error_message in str(err):
+                    is_retriable_error = True
+            if is_retriable_error:
+                self.logger.warning(
+                    f"Failed to create order ({html_util.get_html_summary_if_relevant(err)}) : "
+                    f"order_type: {order_type}, symbol: {symbol}. Retrying order creation."
+                )
+                return await self._create_specific_order(order_type, symbol, quantity, price=price,
+                                                         stop_price=stop_price, side=side,
+                                                         current_price=current_price, reduce_only=reduce_only,
+                                                         params=params)
+            # not retriable, raise
+            raise
         except (ccxt.InvalidOrder, ccxt.BadRequest) as err:
             if self.is_exchange_account_traded_symbol_permission_error(err):
                 # exchange won't let this order create: raise
@@ -386,13 +409,18 @@ class RestExchange(abstract_exchange.AbstractExchange):
                 ) from err
             if self.is_missing_funds_error(err):
                 self._on_missing_funds_err(err, order_type, symbol, quantity, price, stop_price)
+            if not self._enable_create_order_retrier:
+                # should not retry, raise
+                raise
             # can be raised when exchange precision/limits rules change
             self.logger.warning(
                 f"Failed to create order ({html_util.get_html_summary_if_relevant(err)}) : "
                 f"order_type: {order_type}, symbol: {symbol}. "
                 f"This might be due to an update on {self.name} market rules. Fetching updated rules."
             )
-            await self.connector.load_symbol_markets(reload=True, market_filter=self.exchange_manager.market_filter)
+            await self.connector.load_symbol_markets(
+                reload=True, market_filter=self.exchange_manager.market_filter
+            )
             # retry order creation with updated markets (ccxt will use the updated market values)
             return await self._create_specific_order(order_type, symbol, quantity, price=price,
                                                      stop_price=stop_price, side=side,
