@@ -15,7 +15,6 @@
 #  License along with this library.
 import contextlib
 import typing
-
 import ccxt
 import trading_backend
 
@@ -28,21 +27,30 @@ import octobot_commons.tentacles_management as tentacles_management
 import octobot_tentacles_manager.api as api
 
 import octobot_trading.enums as enums
+import octobot_trading.errors as errors
 import octobot_trading.constants as constants
 import octobot_trading.exchanges.types as exchanges_types
 import octobot_trading.exchanges.implementations as exchanges_implementations
 import octobot_trading.exchanges.connectors.ccxt.enums as ccxt_enums
+import octobot_trading.exchanges.connectors.ccxt.ccxt_client_util as ccxt_client_util
 import octobot_trading.exchanges.exchange_details as exchange_details
 import octobot_trading.exchanges.exchange_builder as exchange_builder
 
 
-def get_rest_exchange_class(exchange_name, tentacles_setup_config):
-    return search_exchange_class_from_exchange_name(exchanges_types.RestExchange, exchange_name, tentacles_setup_config)
+def get_rest_exchange_class(
+    exchange_name: str, tentacles_setup_config, exchange_config_by_exchange: typing.Optional[dict[str, dict]]
+):
+    return search_exchange_class_from_exchange_name(
+        exchanges_types.RestExchange, exchange_name, tentacles_setup_config, exchange_config_by_exchange
+    )
 
 
-def search_exchange_class_from_exchange_name(exchange_class, exchange_name,
-                                             tentacles_setup_config, enable_default=False):
-    exchange_class = get_exchange_class_from_name(exchange_class, exchange_name,  tentacles_setup_config, enable_default)
+def search_exchange_class_from_exchange_name(
+    exchange_class, exchange_name, tentacles_setup_config, exchange_config_by_exchange, enable_default=False
+):
+    exchange_class = get_exchange_class_from_name(
+        exchange_class, exchange_name,  tentacles_setup_config, exchange_config_by_exchange, enable_default
+    )
     if exchange_class is not None:
         return exchange_class
     if enable_default:
@@ -58,8 +66,10 @@ def search_exchange_class_from_exchange_name(exchange_class, exchange_name,
     return exchanges_implementations.DefaultRestExchange
 
 
-def get_exchange_class_from_name(exchange_parent_class, exchange_name, tentacles_setup_config,
-                                 enable_default_implementation, strict_name_matching=False):
+def get_exchange_class_from_name(
+    exchange_parent_class, exchange_name, tentacles_setup_config, exchange_config_by_exchange,
+    enable_default_implementation, strict_name_matching=False
+):
     for exchange_candidate in tentacles_management.get_all_classes_from_parent(exchange_parent_class):
         try:
             if _is_exchange_candidate_matching(
@@ -74,19 +84,21 @@ def get_exchange_class_from_name(exchange_parent_class, exchange_name, tentacles
             # As we are searching for an exchange_type specific subclass
             # We should ignore classes that raises NotImplementedError
             pass
-    auto_filled_exchanges = _get_auto_filled_exchanges(tentacles_setup_config)
+    auto_filled_exchanges = _get_auto_filled_exchanges(tentacles_setup_config, exchange_config_by_exchange)
     if exchange_name in auto_filled_exchanges:
         return auto_filled_exchanges[exchange_name][0]
     return None
 
 
-def _get_auto_filled_exchanges(tentacles_setup_config):
+def _get_auto_filled_exchanges(tentacles_setup_config, exchange_config_by_exchange: typing.Optional[dict[str, dict]]):
     auto_filled_exchanges = {}
     for exchange_candidate in _get_auto_filled_exchanges_tentacles():
         if tentacles_setup_config is None:
             # tentacles_setup_config is required for auto-filled exchanges
             continue
-        config = api.get_tentacle_config(
+        config = exchange_config_by_exchange[exchange_candidate.get_name()] if (
+            exchange_config_by_exchange and exchange_candidate.get_name() in exchange_config_by_exchange
+        ) else api.get_tentacle_config(
             tentacles_setup_config, exchange_candidate
         )
         for exchange_name in exchange_candidate.supported_autofill_exchanges(config):
@@ -95,7 +107,7 @@ def _get_auto_filled_exchanges(tentacles_setup_config):
 
 
 def get_auto_filled_exchange_names(tentacles_setup_config):
-    return list(_get_auto_filled_exchanges(tentacles_setup_config))
+    return list(_get_auto_filled_exchanges(tentacles_setup_config, None))
 
 
 def _get_auto_filled_exchanges_tentacles():
@@ -110,13 +122,13 @@ async def get_exchange_details(
     exchange_name, is_autofilled, tentacles_setup_config, aiohttp_session
 ) -> exchange_details.ExchangeDetails:
     if is_autofilled:
-        auto_filled_exchanges = _get_auto_filled_exchanges(tentacles_setup_config)
+        auto_filled_exchanges = _get_auto_filled_exchanges(tentacles_setup_config, None)
         if exchange_name in auto_filled_exchanges:
             return await auto_filled_exchanges[exchange_name][0].get_autofilled_exchange_details(
                 aiohttp_session, auto_filled_exchanges[exchange_name][1], exchange_name
             )
     try:
-        exchange = getattr(ccxt, exchange_name)()
+        exchange = ccxt_client_util.ccxt_exchange_class_factory(exchange_name)()
         return exchange_details.ExchangeDetails(
             exchange.id,
             exchange.name,
@@ -207,7 +219,8 @@ def get_enabled_exchanges(config):
 async def get_local_exchange_manager(
     exchange_name: str, exchange_config: dict, tentacles_setup_config,
     is_sandboxed: bool, ignore_config=False, builder=None, use_cached_markets=True,
-    is_broker_enabled: bool = False,
+    is_broker_enabled: bool = False, exchange_config_by_exchange: typing.Optional[dict[str, dict]] = None,
+    disable_unauth_retry: bool = False,
     market_filter: typing.Union[None, typing.Callable[[dict], bool]] = None
 ):
     exchange_type = exchange_config.get(common_constants.CONFIG_EXCHANGE_TYPE, get_default_exchange_type(exchange_name))
@@ -217,8 +230,10 @@ async def get_local_exchange_manager(
     )
     exchange_manager = await builder.use_tentacles_setup_config(tentacles_setup_config) \
         .is_checking_credentials(False) \
+        .disable_unauth_retry(disable_unauth_retry) \
         .is_sandboxed(is_sandboxed) \
         .is_using_exchange_type(exchange_type) \
+        .use_exchange_config_by_exchange(exchange_config_by_exchange) \
         .is_exchange_only() \
         .is_rest_only() \
         .is_broker_enabled(is_broker_enabled) \
@@ -228,7 +243,8 @@ async def get_local_exchange_manager(
         .disable_trading_mode() \
         .build()
     try:
-        yield exchange_manager
+        with exchange_error_translator(exchange_manager):
+            yield exchange_manager
     finally:
         # do not log stopping message
         logger = exchange_manager.exchange.connector.logger
@@ -238,6 +254,20 @@ async def get_local_exchange_manager(
         logger.disable(False)
 
 
+@contextlib.contextmanager
+def exchange_error_translator(exchange_manager):
+    try:
+        yield
+    except ccxt.ExchangeError as err:
+        # convert permission and compliancy errors while the exchange manager still exists and can be used
+        if exchange_manager.exchange.is_api_permission_error(err):
+            raise errors.AuthenticationError(f"{err} ({err.__class__})") from err
+        if exchange_manager.exchange.is_exchange_rules_compliancy_error(err):
+            raise errors.ExchangeCompliancyError(f"{err} ({err.__class__})") from err
+        # default raise
+        raise
+
+
 async def is_compatible_account(exchange_name: str, exchange_config: dict, tentacles_setup_config, is_sandboxed: bool) \
         -> (bool, bool, str):
     """
@@ -260,6 +290,12 @@ async def is_compatible_account(exchange_name: str, exchange_config: dict, tenta
                 return True, True, None
         except trading_backend.TimeSyncError:
             return False, False, _get_time_sync_error_message(exchange_name, "backend.is_valid_account")
+        except trading_backend.APIKeyPermissionsError as err:
+            return False, False, f"Please update your API Key permissions: {err}"
+        except trading_backend.APIKeyIPWhitelistError as err:
+            return False, False, f"Please update your API Key IP whitelist: {err}"
+        except trading_backend.UnexpectedError as err:
+            return False, False, f"Impossible to check API key: {err}"
         except trading_backend.ExchangeAuthError:
             message = f"Invalid {exchange_name.capitalize()} authentication details"
             if is_sandboxed:
@@ -268,8 +304,6 @@ async def is_compatible_account(exchange_name: str, exchange_config: dict, tenta
                           f"{exchange_name.capitalize()} to trade and validate your api key. " \
                           f"Disable sandbox in your accounts configuration if this is not intended."
             return False, False, message
-        except trading_backend.APIKeyPermissionsError as err:
-            return False, False, f"Please update your API Key permissions: {err}"
         except (AttributeError, Exception) as e:
             return True, False, f"Error when loading exchange account: {e}"
 
@@ -287,6 +321,7 @@ async def get_historical_ohlcv(
     """
     reached_max = False
     time_frame_sec = common_enums.TimeFramesMinutes[time_frame] * common_constants.MINUTE_TO_SECONDS
+    time_frame_msec = time_frame_sec * common_constants.MSECONDS_TO_SECONDS
     exchange_time = local_exchange_manager.exchange.get_exchange_current_time()
     max_theoretical_time = exchange_time - exchange_time % time_frame_sec
     while start_time < end_time and not reached_max:
@@ -308,6 +343,9 @@ async def get_historical_ohlcv(
                 start_time += 1
             else:
                 reached_max = True
+        elif local_exchange_manager.exchange.MAX_FETCHED_OHLCV_COUNT:
+            # history needs to be fetched step by step
+            start_time = start_time + (time_frame_msec * local_exchange_manager.exchange.MAX_FETCHED_OHLCV_COUNT)
         else:
             reached_max = True
 
@@ -328,9 +366,10 @@ def get_default_exchange_type(exchange_name):
     return common_constants.DEFAULT_EXCHANGE_TYPE
 
 
-def get_supported_exchange_types(exchange_name, tentacles_setup_config):
+def get_supported_exchange_types(exchange_name, tentacles_setup_config, exchange_config_by_exchange=None):
     exchange_class = get_exchange_class_from_name(
-        exchanges_types.RestExchange, exchange_name, tentacles_setup_config, False, strict_name_matching=True
+        exchanges_types.RestExchange, exchange_name, tentacles_setup_config,
+        exchange_config_by_exchange, False, strict_name_matching=True
     )
     if exchange_class is None:
         # default
@@ -359,7 +398,7 @@ def update_raw_order_from_raw_trade(order_to_update, raw_trade):
         enums.ExchangeConstantsOrderColumns.PRICE.value)
     order_to_update[enums.ExchangeConstantsOrderColumns.TIMESTAMP.value] = order_to_update.get(
         enums.ExchangeConstantsOrderColumns.TIMESTAMP.value,
-        raw_trade[enums.ExchangeConstantsOrderColumns.TIMESTAMP.value])
+        raw_trade.get(enums.ExchangeConstantsOrderColumns.TIMESTAMP.value))
     order_to_update[enums.ExchangeConstantsOrderColumns.STATUS.value] = enums.OrderStatus.FILLED.value
     order_to_update[enums.ExchangeConstantsOrderColumns.FILLED.value] = raw_trade.get(
         enums.ExchangeConstantsOrderColumns.AMOUNT.value)
@@ -369,6 +408,12 @@ def update_raw_order_from_raw_trade(order_to_update, raw_trade):
     order_to_update[
         enums.ExchangeConstantsOrderColumns.FEE.value
     ] = raw_trade.get(enums.ExchangeConstantsOrderColumns.FEE.value)
+    order_to_update[
+        enums.ExchangeConstantsOrderColumns.TAG.value
+    ] = raw_trade.get(enums.ExchangeConstantsOrderColumns.TAG.value)
+    order_to_update[
+        enums.ExchangeConstantsOrderColumns.REDUCE_ONLY.value
+    ] = raw_trade.get(enums.ExchangeConstantsOrderColumns.REDUCE_ONLY.value, False)
     return order_to_update
 
 
@@ -426,3 +471,11 @@ def get_associated_symbol(exchange_manager, asset: str, target_asset: str) -> (t
         symbol = reversed_symbol
         is_reversed_symbol = True
     return symbol, is_reversed_symbol
+
+
+def is_error_on_this_type(error: BaseException, descriptions: typing.List[typing.Iterable[str]]) -> bool:
+    lower_error = str(error).lower()
+    for identifiers in descriptions:
+        if all(identifier in lower_error for identifier in identifiers):
+            return True
+    return False
