@@ -72,6 +72,7 @@ from octobot_trading.modes.modes_util import (
     get_instantly_filled_limit_order_adapted_price,
     get_instantly_filled_limit_order_adapted_price_and_quantity,
     notify_portfolio_optimization_complete,
+    get_trading_modes_of_this_type_on_this_matrix,
 )
 
 __all__ = [
@@ -102,4 +103,5 @@ __all__ = [
     "get_instantly_filled_limit_order_adapted_price",
     "get_instantly_filled_limit_order_adapted_price_and_quantity",
     "notify_portfolio_optimization_complete",
+    "get_trading_modes_of_this_type_on_this_matrix",
 ]
