@@ -17,6 +17,7 @@ import asyncio
 import typing
 import contextlib
 import decimal
+import copy
 
 import octobot_commons.logging as logging
 
@@ -40,6 +41,7 @@ class Order(util.Initializable):
     """
     CHECK_ORDER_STATUS_AFTER_INIT_DELAY = 2
     SUPPORTS_GROUPING = True    # False when orders of this type can't be grouped
+    USE_ORIGIN_QUANTITY_AS_FILLED_QUANTITY = False
 
     def __init__(self, trader, side=None):
         super().__init__()
@@ -241,7 +243,8 @@ class Order(util.Initializable):
                 should_update_total_cost = True
 
         if self.trader.simulate:
-            if quantity and self.filled_quantity != quantity:
+            if quantity and (self.is_filled() or self.USE_ORIGIN_QUANTITY_AS_FILLED_QUANTITY):
+                # when simulating, only set filled quantity if the order is filled
                 self.filled_quantity = quantity
                 changed = True
                 should_update_total_cost = True
@@ -544,17 +547,23 @@ class Order(util.Initializable):
                 f"when creating active order from inactive order: {self}"
             )
 
-    async def on_pending_creation(self, is_from_exchange_data=False, enable_associated_orders_creation=True):
+    async def on_pending_creation(self, is_from_exchange_data=False, enable_associated_orders_creation=True,
+        is_already_counted_in_available_funds=False
+    ):
         with self.order_state_creation():
             state_class = orders_states.PendingCreationChainedOrderState if self.is_waiting_for_chained_trigger \
                 else orders_states.PendingCreationOrderState
             self.state = state_class(
                 self, is_from_exchange_data=is_from_exchange_data,
-                enable_associated_orders_creation=enable_associated_orders_creation
+                enable_associated_orders_creation=enable_associated_orders_creation,
+                is_already_counted_in_available_funds=is_already_counted_in_available_funds
             )
             await self.state.initialize()
 
-    async def on_open(self, force_open=False, is_from_exchange_data=False, enable_associated_orders_creation=True):
+    async def on_open(
+        self, force_open=False, is_from_exchange_data=False, enable_associated_orders_creation=True,
+        is_already_counted_in_available_funds=False
+    ):
         with self.order_state_creation():
             if isinstance(self.state, orders_states.PendingCreationOrderState):
                 await self.state.trigger_terminate()
@@ -566,11 +575,15 @@ class Order(util.Initializable):
                 return
             self.state = orders_states.OpenOrderState(
                 self, is_from_exchange_data=is_from_exchange_data,
-                enable_associated_orders_creation=enable_associated_orders_creation
+                enable_associated_orders_creation=enable_associated_orders_creation,
+                is_already_counted_in_available_funds=is_already_counted_in_available_funds
             )
             await self.state.initialize(forced=force_open)
 
-    async def on_fill(self, force_fill=False, is_from_exchange_data=False, enable_associated_orders_creation=None):
+    async def on_fill(
+        self, force_fill=False, is_from_exchange_data=False, enable_associated_orders_creation=None,
+        is_already_counted_in_available_funds=False
+    ):
         enable_associated_orders_creation = self.state.enable_associated_orders_creation \
             if (self.state and enable_associated_orders_creation is None) \
             else (True if enable_associated_orders_creation is None else enable_associated_orders_creation)
@@ -581,7 +594,8 @@ class Order(util.Initializable):
             with self.order_state_creation():
                 self.state = orders_states.FillOrderState(
                     self, is_from_exchange_data=is_from_exchange_data,
-                    enable_associated_orders_creation=enable_associated_orders_creation
+                    enable_associated_orders_creation=enable_associated_orders_creation,
+                    is_already_counted_in_available_funds=is_already_counted_in_available_funds
                 )
                 await self.state.initialize(forced=force_fill)
         else:
@@ -589,20 +603,23 @@ class Order(util.Initializable):
                                                              f"or canceled order: "
                                                              f"ignored fill call for {self}")
 
-    async def on_close(self, force_close=False, is_from_exchange_data=False, enable_associated_orders_creation=None):
+    async def on_close(self, force_close=False, is_from_exchange_data=False, enable_associated_orders_creation=None,
+        is_already_counted_in_available_funds=False
+    ):
         enable_associated_orders_creation = self.state.enable_associated_orders_creation \
             if (self.state and enable_associated_orders_creation is None) \
             else (True if enable_associated_orders_creation is None else enable_associated_orders_creation)
         with self.order_state_creation():
             self.state = orders_states.CloseOrderState(
                 self, is_from_exchange_data=is_from_exchange_data,
-                enable_associated_orders_creation=enable_associated_orders_creation
+                enable_associated_orders_creation=enable_associated_orders_creation,
+                is_already_counted_in_available_funds=is_already_counted_in_available_funds
             )
             await self.state.initialize(forced=force_close)
 
     async def on_cancel(
         self, is_from_exchange_data=False, force_cancel=False, enable_associated_orders_creation=None,
-        ignored_order=None
+        ignored_order=None, is_already_counted_in_available_funds=False
     ):
         enable_associated_orders_creation = self.state.enable_associated_orders_creation \
             if (self.state and enable_associated_orders_creation is None) \
@@ -610,7 +627,8 @@ class Order(util.Initializable):
         with self.order_state_creation():
             self.state = orders_states.CancelOrderState(
                 self, is_from_exchange_data=is_from_exchange_data,
-                enable_associated_orders_creation=enable_associated_orders_creation
+                enable_associated_orders_creation=enable_associated_orders_creation,
+                is_already_counted_in_available_funds=is_already_counted_in_available_funds
             )
             await self.state.initialize(forced=force_cancel, ignored_order=ignored_order)
 
@@ -782,6 +800,9 @@ class Order(util.Initializable):
         # consider worse case taker fees when using use_origin_quantity_and_price as the order is not filled yet
         taker_or_maker = enums.ExchangeConstantsOrderColumns.TAKER.value \
             if use_origin_quantity_and_price else self.taker_or_maker
+        if self.is_cleared():
+            # order is cleared, it might have been filled or cancelled. Use existing fees
+            return copy.copy(self.fee)
         if self.fees_currency_side is enums.FeesCurrencySide.UNDEFINED:
             computed_fee = self.exchange_manager.exchange.get_trade_fee(
                 self.symbol, self.order_type, quantity, price, taker_or_maker
@@ -828,7 +849,9 @@ class Order(util.Initializable):
     def generate_executed_time(self):
         return self.exchange_manager.exchange.get_exchange_current_time()
 
-    def is_counted_in_available_funds(self):
+    def is_counted_in_available_funds(self) -> bool:
+        if self.state and self.state.is_already_counted_in_available_funds:
+            return False
         if self.is_active:
             if self.trader.simulate or (
                 self.reduce_only and self.exchange_manager and self.exchange_manager.is_future
@@ -856,6 +879,20 @@ class Order(util.Initializable):
     def is_short(self):
         return self.side is enums.TradeOrderSide.SELL
 
+    def is_partially_filled(self) -> bool:
+        return constants.ZERO < self.filled_quantity < self.origin_quantity
+
+    def get_remaining_quantity(self) -> decimal.Decimal:
+        return self.origin_quantity - self.filled_quantity
+
+    def get_locked_quantity(self) -> decimal.Decimal:
+        return self.filled_quantity if (
+            self.is_filled() or self.USE_ORIGIN_QUANTITY_AS_FILLED_QUANTITY
+        ) else self.get_remaining_quantity()
+
+    def get_remaining_cost(self) -> decimal.Decimal:
+        return self.get_cost(self.get_remaining_quantity())
+
     def update_from_raw(self, raw_order):
         if self.side is None or self.order_type is None:
             try:
@@ -995,13 +1032,16 @@ class Order(util.Initializable):
     def _update_total_cost(self):
         # use filled amounts when available
         quantity = self.filled_quantity if self.filled_quantity else self.origin_quantity
+        self.total_cost = self.get_cost(quantity)
+
+    def get_cost(self, quantity: decimal.Decimal) -> decimal.Decimal:
         price = self.filled_price if self.filled_price else self.origin_price
         if self.quantity_currency == self.currency:
             # quantity in BTC for BTC/USDT => cost = BTC * price(BTC in USDT)
-            self.total_cost = quantity * price
+            return quantity * price
         else:
             # quantity in USDT for BTC/USDT => cost = price(BTC in USDT)
-            self.total_cost = quantity
+            return quantity
 
     def update_order_filled_values(self, ideal_price: decimal.Decimal):
         if not self.filled_price:
@@ -1142,7 +1182,7 @@ class Order(util.Initializable):
         return self.trader is not None
 
 
-def parse_order_type(raw_order):
+def parse_order_type(raw_order) -> (enums.TradeOrderSide, enums.TraderOrderType):
     try:
         side: enums.TradeOrderSide = enums.TradeOrderSide(raw_order[enums.ExchangeConstantsOrderColumns.SIDE.value])
         order_type: enums.TradeOrderType = enums.TradeOrderType.UNKNOWN
