@@ -15,13 +15,17 @@
 #  License along with this library.
 import decimal
 
+import mock
 from mock import Mock, AsyncMock
 import pytest
 
 import octobot_trading.api as api
 import octobot_trading.enums as enums
 import octobot_trading.constants as constants
+import octobot_trading.errors
 import octobot_trading.personal_data as personal_data
+import octobot_trading.personal_data.orders.order_util as order_util
+import octobot_trading.personal_data.orders.order_factory as order_factory
 
 from tests import event_loop
 from tests.exchanges import future_simulated_exchange_manager, simulated_exchange_manager, set_future_exchange_fees
@@ -323,16 +327,78 @@ async def test_get_futures_max_order_size(future_trader_simulator_with_default_l
            (l1_current_symbol_holding * default_contract.current_leverage).quantize(compare_accuracy)
     assert l2_market_quantity.quantize(compare_accuracy) == \
            (l1_market_quantity * default_contract.current_leverage).quantize(compare_accuracy)
-    assert personal_data.get_futures_max_order_size(
+    full_buy_size, increasing = personal_data.get_futures_max_order_size(
         exchange_manager_inst, symbol, enums.TradeOrderSide.BUY,
         decimal.Decimal("37000"), False, l2_current_symbol_holding, l2_market_quantity
-    ) == (decimal.Decimal('0.1346503937177512307045985802'), True)
+    )
+    assert (full_buy_size, increasing) == (decimal.Decimal('0.1346503937177512307045985802'), True)
+
+    # reduce only (position is empty)
+    assert personal_data.get_futures_max_order_size(
+        exchange_manager_inst, symbol, enums.TradeOrderSide.BUY,
+        decimal.Decimal("37000"), True, l2_current_symbol_holding, l2_market_quantity
+    ) == (constants.ZERO, False)
 
     # with short position
-    assert personal_data.get_futures_max_order_size(
+    full_sell_size, increasing = personal_data.get_futures_max_order_size(
         exchange_manager_inst, symbol, enums.TradeOrderSide.SELL,
         decimal.Decimal("37000"), False, l2_current_symbol_holding, l2_market_quantity
-    ) == (decimal.Decimal('0.1345431452958334678764786291'), True)
+    )
+    assert (full_sell_size, increasing) == (decimal.Decimal('0.1345431452958334678764786291'), True)
+    # reduce only (position is empty)
+    assert personal_data.get_futures_max_order_size(
+        exchange_manager_inst, symbol, enums.TradeOrderSide.SELL,
+        decimal.Decimal("37000"), True, l2_current_symbol_holding, l2_market_quantity
+    ) == (constants.ZERO, False)
+
+    # LONG POSITION
+    # reduce only (position is NOT empty)
+    position = exchange_manager_inst.exchange_personal_data.positions_manager.get_symbol_positions(symbol)[0]
+    await position.update(mark_price=decimal.Decimal(37000), update_margin=decimal.Decimal(4))
+    # position is now long
+    position_size = position.size
+    assert position_size == decimal.Decimal("0.0005405405405405405405405405405")
+    assert personal_data.get_futures_max_order_size(
+        exchange_manager_inst, symbol, enums.TradeOrderSide.SELL,
+        decimal.Decimal("37000"), True, l2_current_symbol_holding, l2_market_quantity
+    ) == (position_size, False)
+
+    # not reduce only and long position: can reverse position
+    position_size = position.size
+    assert position_size == decimal.Decimal("0.0005405405405405405405405405405")    # still the same size
+    max_size, increasing = personal_data.get_futures_max_order_size(
+        exchange_manager_inst, symbol, enums.TradeOrderSide.SELL,
+        decimal.Decimal("37000"), False, l2_current_symbol_holding, l2_market_quantity
+    )
+    assert increasing is True
+    assert max_size > position_size
+    # higher than when just opening a position with full free funds
+    assert max_size > full_sell_size
+    assert max_size == decimal.Decimal("0.1352986545732659722297027996")
+
+    # SHORT POSITION
+    # reduce only (position is NOT empty)
+    await position.update(mark_price=decimal.Decimal(37000), update_margin=decimal.Decimal(-500))
+    # position is now short
+    position_size = position.size
+    assert position_size == decimal.Decimal("-0.06702702702702702702702702703")
+    assert personal_data.get_futures_max_order_size(
+        exchange_manager_inst, symbol, enums.TradeOrderSide.BUY,
+        decimal.Decimal("37000"), True, l2_current_symbol_holding, l2_market_quantity
+    ) == (-position_size, False)
+
+    # not reduce only and short position: can reverse position
+    position_size = position.size
+    assert position_size == decimal.Decimal("-0.06702702702702702702702702703")    # still the same size
+    max_size, increasing = personal_data.get_futures_max_order_size(
+        exchange_manager_inst, symbol, enums.TradeOrderSide.BUY,
+        decimal.Decimal("37000"), False, l2_current_symbol_holding, l2_market_quantity
+    )
+    assert increasing is True
+    assert max_size > abs(position_size)
+    # higher than when just opening a position with full free funds
+    assert max_size > full_sell_size
+    assert max_size == decimal.Decimal("0.2149168523362071749168523362")
 
 
 @pytest.mark.asyncio
@@ -346,16 +412,25 @@ async def test_create_as_chained_order_regular_order(trader_simulator):
                       quantity=decimal.Decimal("10"),
                       price=decimal.Decimal("70"))
     base_order.is_waiting_for_chained_trigger = True
+    base_order.creation_time = 123
+    origin_time = base_order.creation_time
+    assert base_order.to_dict()[enums.ExchangeConstantsOrderColumns.TIMESTAMP.value] \
+           == origin_time
     # base_order.state is None since no state has been associated to it (not initiliazed)
     assert base_order.state is None
     assert base_order.is_initialized is False
 
     await personal_data.create_as_chained_order(base_order)
+    assert base_order.creation_time > origin_time   # creation_time got update using current time
     assert base_order.is_waiting_for_chained_trigger is False
     assert base_order.is_created() is True
     assert base_order.is_initialized is True
     assert isinstance(base_order.state, personal_data.OpenOrderState)
 
+    # use chained order creation time
+    assert base_order.to_dict()[enums.ExchangeConstantsOrderColumns.TIMESTAMP.value] \
+           == base_order.creation_time
+
 
 @pytest.mark.asyncio
 async def test_create_as_chained_order_bundled_order_no_open_order(trader_simulator):
@@ -443,6 +518,104 @@ async def test_create_as_chained_order_bundled_order_no_open_order(trader_simula
     assert open_order_2.exchange_manager is None
 
 
+@pytest.mark.asyncio
+async def test_create_as_active_order_using_strategy_if_any(trader_simulator):
+    config, exchange_manager_inst, trader_inst = trader_simulator
+    inactive_order = personal_data.SellLimitOrder(trader_inst)
+    group = personal_data.OneCancelsTheOtherOrderGroup(
+        "plop", exchange_manager_inst.exchange_personal_data.orders_manager
+    )
+
+    with mock.patch.object(order_util, "create_as_active_order_on_exchange", mock.AsyncMock()) as create_as_active_order_on_exchange, \
+        mock.patch.object(group.active_order_swap_strategy.__class__, "execute", mock.AsyncMock()) as execute_mock:
+        # no strategy
+        await personal_data.create_as_active_order_using_strategy_if_any(inactive_order, 123, "callback")
+        create_as_active_order_on_exchange.assert_called_once_with(inactive_order, False)
+        execute_mock.assert_not_called()
+        create_as_active_order_on_exchange.reset_mock()
+
+        # with strategy
+        inactive_order.order_group = group
+        await personal_data.create_as_active_order_using_strategy_if_any(inactive_order, 123, "callback")
+        create_as_active_order_on_exchange.assert_not_called()
+        execute_mock.assert_called_once_with(inactive_order, "callback", 123)
+
+
+@pytest.mark.asyncio
+async def test_create_as_active_order_on_exchange_with_sub_calls(trader_simulator):
+    config, exchange_manager_inst, trader_inst = trader_simulator
+    trader_inst.simulate = False
+    trader_inst.enable_inactive_orders = True
+    inactive_order = personal_data.SellLimitOrder(trader_inst)
+    inactive_order.is_active = False
+    exchange_created_order = personal_data.SellLimitOrder(trader_inst)
+    exchange_created_order.update(
+        order_type=enums.TraderOrderType.SELL_LIMIT,
+        order_id="base_order_id",
+        symbol="BTC/USDT",
+        current_price=decimal.Decimal("55"),
+        quantity=decimal.Decimal("10"),
+        price=decimal.Decimal("70"),
+        reduce_only=False,
+        fee={
+            enums.FeePropertyColumns.COST.value: 1,
+            enums.FeePropertyColumns.CURRENCY.value: "USDT",
+        }
+    )
+    await exchange_manager_inst.exchange_personal_data.orders_manager.upsert_order_instance(inactive_order)
+    all_orders = exchange_manager_inst.exchange_personal_data.orders_manager.get_all_orders()
+    assert all_orders == [inactive_order]
+    with mock.patch.object(exchange_manager_inst.exchange, "create_order", mock.AsyncMock(return_value=exchange_created_order)) as create_order_mock, \
+        mock.patch.object(order_factory, "create_order_instance_from_raw", mock.Mock(return_value=exchange_created_order)) as create_order_instance_from_raw_mock:
+        active_order = await personal_data.create_as_active_order_on_exchange(inactive_order, True)
+        assert active_order is exchange_created_order
+        assert active_order.is_active is True
+        all_orders = exchange_manager_inst.exchange_personal_data.orders_manager.get_all_orders()
+        # inactive order is replaced by active_order
+        assert all_orders == [active_order]
+        # inactive_order is now active and cleared
+        assert inactive_order.is_active is True
+        assert inactive_order.exchange_manager is None
+        create_order_mock.assert_called_once()
+        create_order_instance_from_raw_mock.assert_called_once()
+
+
+@pytest.mark.asyncio
+async def test_update_order_as_inactive_on_exchange_with_sub_calls(trader_simulator):
+    config, exchange_manager_inst, trader_inst = trader_simulator
+    trader_inst.enable_inactive_orders = True
+    trader_inst.simulate = False
+    active_order = personal_data.SellLimitOrder(trader_inst)
+    active_order.update(
+        order_type=enums.TraderOrderType.SELL_LIMIT,
+        order_id="base_order_id",
+        symbol="BTC/USDT",
+        current_price=decimal.Decimal("55"),
+        quantity=decimal.Decimal("10"),
+        price=decimal.Decimal("70"),
+        reduce_only=False,
+        active_trigger=personal_data.create_order_price_trigger(active_order, decimal.Decimal("70"), False),
+        fee={
+            enums.FeePropertyColumns.COST.value: 1,
+            enums.FeePropertyColumns.CURRENCY.value: "USDT",
+        }
+    )
+    await exchange_manager_inst.exchange_personal_data.orders_manager.upsert_order_instance(active_order)
+    all_orders = exchange_manager_inst.exchange_personal_data.orders_manager.get_all_orders()
+    assert all_orders == [active_order]
+    assert active_order.active_trigger._trigger_event is None
+    with mock.patch.object(exchange_manager_inst.exchange, "cancel_order", mock.AsyncMock(return_value=enums.OrderStatus.CANCELED)) as cancel_order_mock:
+        assert await personal_data.update_order_as_inactive_on_exchange(active_order, False) is True
+        assert exchange_manager_inst.exchange_personal_data.orders_manager.get_all_orders() == [active_order]
+        # order is now inactive
+        assert exchange_manager_inst.exchange_personal_data.orders_manager.get_all_orders(active=False) == [active_order]
+        assert active_order.is_active is False
+        assert active_order.status == enums.OrderStatus.OPEN
+        assert active_order.active_trigger is not None
+        assert active_order.active_trigger._trigger_event is not None
+        cancel_order_mock.assert_called_once()
+
+
 @pytest.mark.asyncio
 async def test_ensure_orders_relevancy_without_positions(trader_simulator):
     config, exchange_manager_inst, trader_inst = trader_simulator
@@ -528,6 +701,184 @@ async def test_get_order_size_portfolio_percent(trader_simulator):
     ) == decimal.Decimal("10")
 
 
+def test_get_valid_split_orders():
+    # example SOL/BTC values from 12/09/2023
+    symbol_market = {
+        enums.ExchangeConstantsMarketStatusColumns.LIMITS.value: {
+            enums.ExchangeConstantsMarketStatusColumns.LIMITS_AMOUNT.value: {
+                enums.ExchangeConstantsMarketStatusColumns.LIMITS_AMOUNT_MIN.value: 0.01,
+                enums.ExchangeConstantsMarketStatusColumns.LIMITS_AMOUNT_MAX.value: 90000000.0,
+            },
+            enums.ExchangeConstantsMarketStatusColumns.LIMITS_COST.value: {
+                enums.ExchangeConstantsMarketStatusColumns.LIMITS_COST_MIN.value: 0.0001,
+                enums.ExchangeConstantsMarketStatusColumns.LIMITS_COST_MAX.value: 9000000.0
+            },
+            enums.ExchangeConstantsMarketStatusColumns.LIMITS_PRICE.value: {
+                enums.ExchangeConstantsMarketStatusColumns.LIMITS_PRICE_MIN.value: 1e-07,
+                enums.ExchangeConstantsMarketStatusColumns.LIMITS_PRICE_MAX.value: 1000.0
+            },
+        },
+        enums.ExchangeConstantsMarketStatusColumns.PRECISION.value: {
+            enums.ExchangeConstantsMarketStatusColumns.PRECISION_PRICE.value: 7,
+            enums.ExchangeConstantsMarketStatusColumns.PRECISION_AMOUNT.value: 2
+        }
+    }
+    # all valid values
+    prices = [
+        decimal.Decimal("0.0006858"),
+        decimal.Decimal("0.0006908"),
+        decimal.Decimal("0.0006938"),
+        decimal.Decimal("0.0006958"),
+    ]
+    assert personal_data.get_valid_split_orders(
+        decimal.Decimal("1"),
+        prices,
+        symbol_market
+    ) == ([decimal.Decimal("1") / len(prices)] * len(prices), prices)
+
+    # too small amount for cost: adapt to 3 orders
+    assert personal_data.get_valid_split_orders(
+        decimal.Decimal("0.5"),
+        prices,
+        symbol_market
+    ) == ([decimal.Decimal("0.5") / 3] * 3, prices[:3])
+
+    # too small amount for cost: adapt to 1 orders
+    assert personal_data.get_valid_split_orders(
+        decimal.Decimal("0.1"),
+        prices,
+        symbol_market
+    ) == ([decimal.Decimal('0.1')], [decimal.Decimal('0.0006858')])
+
+    # too small amount for cost (because of the lowest price order amount that gets truncated because of
+    # exchange precision rules): adapt to 0 orders
+
+    prices = [
+        decimal.Decimal("0.0006963"),
+        decimal.Decimal("0.0006908"),
+        decimal.Decimal("0.0006938"),
+        decimal.Decimal("0.0006958"),
+    ]
+    assert personal_data.get_valid_split_orders(
+        decimal.Decimal("0.14985"),
+        prices,
+        symbol_market,
+    ) == ([decimal.Decimal('0.14985')], [decimal.Decimal('0.0006963')])
+
+
+def test_get_valid_split_orders_with_amount_ratio_per_order():
+    # example SOL/BTC values from 12/09/2023
+    symbol_market = {
+        enums.ExchangeConstantsMarketStatusColumns.LIMITS.value: {
+            enums.ExchangeConstantsMarketStatusColumns.LIMITS_AMOUNT.value: {
+                enums.ExchangeConstantsMarketStatusColumns.LIMITS_AMOUNT_MIN.value: 0.01,
+                enums.ExchangeConstantsMarketStatusColumns.LIMITS_AMOUNT_MAX.value: 90000000.0,
+            },
+            enums.ExchangeConstantsMarketStatusColumns.LIMITS_COST.value: {
+                enums.ExchangeConstantsMarketStatusColumns.LIMITS_COST_MIN.value: 0.0001,
+                enums.ExchangeConstantsMarketStatusColumns.LIMITS_COST_MAX.value: 9000000.0
+            },
+            enums.ExchangeConstantsMarketStatusColumns.LIMITS_PRICE.value: {
+                enums.ExchangeConstantsMarketStatusColumns.LIMITS_PRICE_MIN.value: 1e-07,
+                enums.ExchangeConstantsMarketStatusColumns.LIMITS_PRICE_MAX.value: 1000.0
+            },
+        },
+        enums.ExchangeConstantsMarketStatusColumns.PRECISION.value: {
+            enums.ExchangeConstantsMarketStatusColumns.PRECISION_PRICE.value: 7,
+            enums.ExchangeConstantsMarketStatusColumns.PRECISION_AMOUNT.value: 2
+        }
+    }
+
+    # all valid values
+    input_prices = [
+        decimal.Decimal("0.0006858"),
+        decimal.Decimal("0.0006908"),
+        decimal.Decimal("0.0006938"),
+        decimal.Decimal("0.0006958"),
+    ]
+    amount_ratio_per_order = [
+        decimal.Decimal("0.12"),
+        decimal.Decimal("0.24"),
+        decimal.Decimal("0.36"),
+        decimal.Decimal("0.28"),
+    ]
+    # invalid inputs
+    with pytest.raises(ValueError):
+        personal_data.get_valid_split_orders(
+            decimal.Decimal("2"),
+            input_prices,
+            symbol_market,
+            amount_ratio_per_order=[1, 2, 3]    # not enough values
+        )
+    with pytest.raises(ValueError):
+        personal_data.get_valid_split_orders(
+            decimal.Decimal("2"),
+            [decimal.Decimal("0.5")],
+            symbol_market,
+            amount_ratio_per_order=[decimal.Decimal(0)]    # 0 values
+        )
+    with pytest.raises(ValueError):
+        personal_data.get_valid_split_orders(
+            decimal.Decimal("2"),
+            [decimal.Decimal("0.5")],
+            symbol_market,
+            amount_ratio_per_order=[decimal.Decimal(-1.2)]    # negative values
+        )
+
+    # valid inputs
+    volumes, prices = personal_data.get_valid_split_orders(
+        decimal.Decimal("2"),
+        input_prices,
+        symbol_market,
+        amount_ratio_per_order=amount_ratio_per_order
+    )
+    assert (volumes, prices) == (
+        [decimal.Decimal("2") * amount_ratio for amount_ratio in amount_ratio_per_order],
+        prices
+    )
+    assert sum(volumes) == decimal.Decimal("2")
+
+    # too small amount for cost: adapt to 2 orders
+    amount_ratio_per_order = [decimal.Decimal(33), decimal.Decimal(44), decimal.Decimal(55), decimal.Decimal(66)]
+    total_ratio = sum(amount_ratio_per_order[:2])
+    volumes, prices = personal_data.get_valid_split_orders(
+        decimal.Decimal("0.5"),
+        input_prices,
+        symbol_market,
+        amount_ratio_per_order=amount_ratio_per_order
+    )
+    assert (volumes, prices) == (
+        [decimal.Decimal("0.5") * amount_ratio/total_ratio for amount_ratio in amount_ratio_per_order[:2]],
+        prices
+    )
+    assert sum(volumes) == decimal.Decimal("0.5")
+
+    # too small amount for cost: adapt to 1 order
+    volumes, prices = personal_data.get_valid_split_orders(
+        decimal.Decimal("0.1"),
+        input_prices,
+        symbol_market,
+        amount_ratio_per_order=amount_ratio_per_order
+    )
+    assert (volumes, prices) == ([decimal.Decimal('0.1')], [decimal.Decimal('0.0006858')])
+
+    # too small amount for cost (because of the lowest price order amount that gets truncated because of
+    # exchange precision rules): adapt to 0 orders
+
+    input_prices = [
+        decimal.Decimal("0.0006963"),
+        decimal.Decimal("0.0006908"),
+        decimal.Decimal("0.0006938"),
+        decimal.Decimal("0.0006958"),
+    ]
+    assert personal_data.get_valid_split_orders(
+        decimal.Decimal("0.14985"),
+        input_prices,
+        symbol_market,
+        amount_ratio_per_order=amount_ratio_per_order
+    ) == ([decimal.Decimal('0.14985')], [decimal.Decimal('0.0006963')])
+
+
 def test_get_split_orders_count_and_increment():
     # example SOL/BTC values from 12/09/2023
     symbol_market = {
@@ -590,3 +941,56 @@ def test_get_split_orders_count_and_increment():
         symbol_market,
         False
     ) == (0, decimal.Decimal('0'))
+
+
+def test_ensure_orders_limit():
+    symbol = "BTC/USDT"
+    exchange_manager = mock.Mock(
+        exchange = mock.Mock(
+            get_max_orders_count = mock.Mock(return_value=5),
+        ),
+        exchange_personal_data = mock.Mock(
+            orders_manager = mock.Mock(
+                get_open_orders = mock.Mock(
+                    return_value=[
+                        mock.Mock(order_type=enums.TraderOrderType.BUY_LIMIT),
+                        mock.Mock(order_type=enums.TraderOrderType.SELL_LIMIT),
+                        mock.Mock(order_type=enums.TraderOrderType.STOP_LOSS),
+                        mock.Mock(order_type=enums.TraderOrderType.STOP_LOSS),
+                        mock.Mock(order_type=enums.TraderOrderType.STOP_LOSS),
+                    ],
+                )
+            )
+        )
+    )
+
+    # do not raise
+    order_util.ensure_orders_limit(exchange_manager, symbol, [])
+    assert exchange_manager.exchange.get_max_orders_count.call_count == 2
+    assert exchange_manager.exchange.get_max_orders_count.mock_calls[0].args == (
+        symbol, enums.TraderOrderType.SELL_LIMIT
+    )
+    assert exchange_manager.exchange.get_max_orders_count.mock_calls[1].args == (
+        symbol, enums.TraderOrderType.STOP_LOSS
+    )
+    exchange_manager.exchange_personal_data.orders_manager.get_open_orders.assert_called_once_with(
+        symbol=symbol, active=None
+    )
+
+    # ok added limit orders
+    order_util.ensure_orders_limit(exchange_manager, symbol, [enums.TraderOrderType.BUY_LIMIT]*2)
+    order_util.ensure_orders_limit(exchange_manager, symbol, [enums.TraderOrderType.BUY_LIMIT]*3)
+    # too many added limit orders
+    with pytest.raises(octobot_trading.errors.MaxOpenOrderReachedForSymbolError):
+        order_util.ensure_orders_limit(exchange_manager, symbol, [enums.TraderOrderType.BUY_LIMIT]*4)
+    with pytest.raises(octobot_trading.errors.MaxOpenOrderReachedForSymbolError):
+        order_util.ensure_orders_limit(exchange_manager, symbol, [enums.TraderOrderType.BUY_LIMIT]*10)
+
+    # ok added stop orders
+    order_util.ensure_orders_limit(exchange_manager, symbol, [enums.TraderOrderType.STOP_LOSS])
+    order_util.ensure_orders_limit(exchange_manager, symbol, [enums.TraderOrderType.STOP_LOSS_LIMIT]*2)
+    # too many added stop orders
+    with pytest.raises(octobot_trading.errors.MaxOpenOrderReachedForSymbolError):
+        order_util.ensure_orders_limit(exchange_manager, symbol, [enums.TraderOrderType.STOP_LOSS]*3)
+    with pytest.raises(octobot_trading.errors.MaxOpenOrderReachedForSymbolError):
+        order_util.ensure_orders_limit(exchange_manager, symbol, [enums.TraderOrderType.STOP_LOSS_LIMIT]*10)
